/*
 * readthread.cpp
 *
 *  Created on: Jul 23, 2020
 *      Author: claudio
 */

#include "switchenergythread.h"
#include "TwinApuId.h"

namespace TwinApuId_ns {

switchenergythread::switchenergythread(Tango::DeviceImpl *devImpl, bool ene) :
						Tango::LogAdapter(devImpl), device(devImpl), abortflag(false), high_energy(ene){
	//DEBUG_STREAM << "switchenergythread::switchenergythread(): constructor << endl;

}

//+------------------------------------------------------------------
//
//      method:                 switchenergythread::run()
//
//      description:    Run
//
//-------------------------------------------------------------------
void* switchenergythread::run_undetached(void*) {
	DEBUG_STREAM << "switchenergythread::run(): running... high_energy="
			<< high_energy << endl;
	struct timeval begin, end;

	TwinApuId *_devicebase = static_cast<TwinApuId*>(device);
	_devicebase->switching_energy = true;
	Tango::DeviceProxy *high_energy_device = _devicebase->high_energy_device;
	Tango::DeviceProxy *low_energy_device = _devicebase->low_energy_device;
	Tango::DeviceProxy *energy_switch_device = _devicebase->energy_switch_device;

	// step 1 : bring APU to injection position
	try {
		double inj_pos_high = _devicebase->highEnergyPeriod / 2.0;
		double inj_pos_low = _devicebase->lowEnergyPeriod / 2.0;
		Tango::DeviceAttribute da_high("UserPosition", inj_pos_high);
		Tango::DeviceAttribute da_low("UserPosition", inj_pos_low);
		high_energy_device->write_attribute(da_high);
		low_energy_device->write_attribute(da_low);
		//force not ready while moving
		*_devicebase->attr_Ready_read = false;
		_devicebase->id_flags[0] = false;
		//attr_StatRF_read = id_flags;
	} catch (Tango::DevFailed &e) {
		ERROR_STREAM << "Failed to set injection position on motor(s) device";
		_devicebase->set_state(Tango::DevState::FAULT);
		_devicebase->set_status(
				"Failed to set injection position on motor(s) device");
		_devicebase->switching_energy=false;
		return 0;

	}

	// wait for termination of movement , maximum time or abortflag

	gettimeofday(&begin, NULL); //tweak initial time

	useconds_t sleep_time = 1000 * _devicebase->readThreadPollTime; //wait one refersh cyslce for checking state
	usleep(2 * sleep_time); // give time for reaction of low level systems end detecting it

	Tango::DevState mstate = _devicebase->_state;
	bool not_moving = (mstate == Tango::DevState::OFF || mstate == Tango::DevState::STANDBY) && (mstate != Tango::DevState::FAULT);

	while ( !not_moving ) //one second loop
	{
		// check forced exit
		if (abortflag){
			_devicebase->switching_energy=false;
			return 0;
		}
		// wait for injection_position  to be reached
		mstate = _devicebase->_state;
		not_moving = (mstate == Tango::DevState::OFF || mstate == Tango::DevState::STANDBY) && (mstate != Tango::DevState::FAULT);
		// timekeeping  and checking
		INFO_STREAM << "switchenergy waiting, not_moving=" << not_moving;
		gettimeofday(&end, NULL);
		time_t diff_time = ((end.tv_sec - begin.tv_sec)
				+ ((time_t) (end.tv_usec - begin.tv_usec)) / 1000000);
		if (diff_time > _devicebase->max_time_to_inj_position) {
			ERROR_STREAM
			<< "switchenergythread : injection_position not reached in time";
			_devicebase->set_state(Tango::DevState::FAULT);
			_devicebase->set_status("injection_position not reached in time");
			_devicebase->switching_energy = false;
			return 0;
		}
		usleep(sleep_time);
	}
	if (abortflag) {
		_devicebase->switching_energy = false;
		return 0;
	} //end immediately on abort
	usleep(2 * sleep_time); // give time for reaction of low level systems end detecting it
	// now switch energy only if ready, otherwise notify...
	mstate = _devicebase->_state;
	not_moving = (mstate == Tango::DevState::OFF || mstate == Tango::DevState::STANDBY ) && (mstate != Tango::DevState::FAULT);
	INFO_STREAM << "switch energy phase2 not_moving=" << not_moving << " state "<<mstate;
	if (mstate != Tango::DevState::FAULT){
		Tango::DevDouble pos;
		if (high_energy)
			pos = _devicebase->highEnergyPosition;
		else
			pos = _devicebase->lowEnergyPosition;
		Tango::DeviceAttribute da("UserPosition", pos);
		try {
			energy_switch_device->write_attribute(da);
			//_devicebase->set_state(Tango::DevState::MOVING);
			*_devicebase->attr_Ready_read = false;
			_devicebase->id_flags[0] = false;
		} catch (Tango::DevFailed &e) {
			Tango::Except::print_exception(e);
			_devicebase->set_state(Tango::DevState::FAULT);
			_devicebase->set_status(
					"failed to write attribute UserPosition on energy_switch_device");
			ERROR_STREAM
			<< "failed to write attribute UserPosition on energy_switch_device";
		}
	} else {
		_devicebase->set_state(Tango::DevState::FAULT);
		_devicebase->set_status(
				"flag READY == false, not ready for energy switch");
		ERROR_STREAM <<"flag READY == false, not ready for energy switch";
		_devicebase->switching_energy=false;
		return 0;
	}
	usleep(2 * sleep_time); // give time for reaction of low level systems end detecting it
	// step 3: wait for energy motor to stop or max time
	not_moving = ( *_devicebase->attr_EnergySwitchAxisState_read == Tango::DevState::OFF || *_devicebase->attr_EnergySwitchAxisState_read == Tango::DevState::STANDBY);
	INFO_STREAM << "switch energy phase3 not_moving=" << not_moving << " state "<<mstate;
	while ( !not_moving)
	{
		// check forced exit
		if (abortflag){
			_devicebase->switching_energy=false;
			return 0;
		}
		// wait for final position to be reached
		not_moving = ( *_devicebase->attr_EnergySwitchAxisState_read == Tango::DevState::OFF || *_devicebase->attr_EnergySwitchAxisState_read == Tango::DevState::STANDBY);
		// timekeeping  and checking
		DEBUG_STREAM << "switchenergy waiting, not_moving=" << not_moving;
		gettimeofday(&end, NULL);
		time_t diff_time = ((end.tv_sec - begin.tv_sec)
				+ ((time_t) (end.tv_usec - begin.tv_usec)) / 1000000);
		DEBUG_STREAM << " times " << diff_time << " " << _devicebase->maxTimeToEnergySwitch;
		if (diff_time > _devicebase->maxTimeToEnergySwitch) {
			ERROR_STREAM
			<< "switchenergythread : injection_position not reached in time";
			_devicebase->set_state(Tango::DevState::FAULT);
			_devicebase->set_status("injection_position not reached in time");
			_devicebase->switching_energy = false;
			return 0;
		}
		usleep(sleep_time);
	} // while

	DEBUG_STREAM << "switchenergythread::run(): exit!!!" << endl;
	_devicebase->switching_energy=false;
	return 0;
}

}
